clear all, close all hidden
load Data     % Load Ps matrix of price scenarios and pT

% -- Portfolio and objective function parameters --
OF      = 'OFVaR';   % Name of objective function
OD.Ps   = Ps;        % Matrix with price scenarios
OD.nA   = size(Ps,2);% Number of assets (columns)
OD.pT   = pT;        % Asset price for planning period T
OD.vT   = 1e7;       % Initial wealth 
OD.vinf = 0.001;     % Minimum holdings (can be a vector)
OD.vsup = 0.070;     % Maximum holdings (can be a vector)
OD.csup = 0.005;     % Maximum weight of cash
OD.K    = 30;        % Maximum cardinality of portfolio
rf   = 0.0001;    % Risk free return
rd   = 0.0010;    % Desired return
beta = 0.05;      % Right tail considered

% -- TA parameters --
TAopt = TAPF('defaults');
TAopt.Restarts =    5;
TAopt.Rounds   =   10;
TAopt.Steps    = 3000;
TAopt.Figures  = 'yes';

start = clock;
% -- Start portfolio optimization --
if TAopt.Restarts > 1
    TAopt.OnlyThresholdSequence = 'yes';
    output = TAPF(OF,TAopt,OD, OD.vT,rd,rf,beta);
    TAopt.ThresholdSequence = output.th;
    TAopt.OnlyThresholdSequence = 'no';
end

for i = 1:TAopt.Restarts
    j = unidrnd(1e6);  % Set random seed for generator
    TAopt.State = j;
    results(i) =  TAPF(OF,TAopt,OD, OD.vT,rd,rf,beta);
end
ttime = etime(clock,start);

if TAopt.Restarts > 1
    % -- Find best result among restarts
    Fbest = realmax; Fvec = NaN(1,TAopt.Restarts); nc=0;
    for i = 1:TAopt.Restarts
        if ~isempty(results(i))
            nc = nc + 1; Fvec(nc) = results(i).F;
            if results(i).F < Fbest
                ibest = i;
                Fbest = results(i).F;
            end
        end
    end
    if strcmp(TAopt.Figures,'yes')
        figure(2),subplot(313)
        h = cdfplot(Fvec);
        set(h,'Color','m','LineWidth',2);
        title('Empirical CDF of solutions');
        set(gca,'FontSize',12); grid on; 
        ylabel(''); xlabel('');
    end
else
    ibest = 1;
end

% -- Results
J     = results(ibest).J;    % Indices of assets in portfolio
x     = results(ibest).x;    % Long vector (1:nA)
cash  = results(ibest).cash;
w     = results(ibest).w;    % Long vector (1:nA)
F     = results(ibest).F;    % Value of objective function

% Plot final portfolio weights and print results
if strcmp(TAopt.Figures,'yes')
    figure(1),subplot(211),hold off
    bar(w,'FaceColor',[0 0 1]), hold on, grid on
    title('Final portfolio weights')
    hb = bar(OD.nA+1,cash/OD.vT,'r');
    set(hb,'Edgecolor','r');
    ylim([0 1.1*max(w)]);
end
fprintf('\n Value of objective function: %7.0f',F);
fprintf('  (%i assets selected)',length(J));
pen = results(i).Pen; % pen = [f(pen1,pen2) pen1 pen2]
if pen(1) ~= 0
    fprintf(['\n Constraints not satisfied:',...
             'return is %6.4f\n'],-pen(2));
end
fprintf('\n Asset  Weight (%%)   Quantity');
for i = 1:length(J)
    j = J(i);
    fprintf('\n %3i    %5.2f        %6i',j,100*w(j),x(j));
end
fprintf('\n');

fprintf('\n %i restarts in %i sec\n',TAopt.Restarts,...
    fix(ttime));
fprintf('   (%i sec per restart)\n',...
    fix(ttime/TAopt.Restarts));

